package org.gidea.scada.monitor.rabbit.listener;

import com.alibaba.fastjson.JSONObject;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.gidea.scada.monitor.rabbit.entity.MonitorMessage;
import org.gidea.scada.monitor.udp.config.UdpServerConfig;
import org.gidea.scada.monitor.udp.entity.CjdMessage;
import org.gidea.scada.monitor.udp.utils.CrcUtils;
import org.gidea.scada.monitor.utils.RabbitExchangeEnum;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.IOException;

@Component
@Log4j2
public class CommandListenerHandler {
    @Resource
    private UdpServerConfig udpServerConfig;
    @Resource
    private Channel channel;

    public void handler(String message) {
        MonitorMessage monitorMessage = JSONObject.parseObject(message, MonitorMessage.class);
        String cjdIp = udpServerConfig.getCjdIp(monitorMessage.getBscode());
        if (cjdIp == null) {
            JSONObject data = new JSONObject();
            data.put("time", System.currentTimeMillis());
            data.put("result", "采集端未连接！");
            monitorMessage.setData(data);
            try {
                channel.basicPublish(RabbitExchangeEnum.MonitorCommandClient.exchangeName, "", null, JSONObject.toJSONString(monitorMessage).getBytes("UTF-8"));
            } catch (IOException e) {
                log.error("响应消息失败！{}", JSONObject.toJSONString(monitorMessage), e);
                throw new RuntimeException(e);
            }
        } else {
            CjdMessage cjdMessage = new CjdMessage();
            BeanUtils.copyProperties(monitorMessage, cjdMessage);
            cjdMessage.setIp(cjdIp);
            cjdMessage.setCode(0);
            String string = JSONObject.toJSONString(cjdMessage);
            String msg = CrcUtils.getCrc(string.getBytes()) + "=" + string;
            try {
                channel.basicPublish(RabbitExchangeEnum.MonitorCjdClient.exchangeName, "", null, msg.getBytes());
            } catch (IOException e) {
                log.error("响应消息失败！{}", msg, e);
                throw new RuntimeException(e);
            }
        }
    }

}
